@@ -157,6 +157,7 @@ options:
         description:
         - a list of ENI IDs (strings) or a list of objects containing the key I(id).
         type: list
+        elements: str
       assign_public_ip:
         description:
         - when true assigns a public IP address to the interface
@@ -169,6 +170,7 @@ options:
         description:
         - a list of IPv6 addresses to assign to the network interface
         type: list
+        elements: str
       source_dest_check:
         description:
         - controls whether source/destination checking is enabled on the interface
@@ -181,6 +183,7 @@ options:
         description:
         - a list of IPv4 addresses to assign to the network interface
         type: list
+        elements: str
       subnet_id:
         description:
         - the subnet to connect the network interface to
@@ -198,6 +201,7 @@ options:
         description:
         - a list of security group IDs to attach to the interface
         type: list
+        elements: str
   volumes:
     description:
     - A list of block device mappings, by default this will always use the AMI root device so the volumes option is primarily for adding more storage.
@@ -1457,6 +1461,8 @@ def change_network_attachments(instance, params):
 
 def find_instances(ids=None, filters=None):
     paginator = client.get_paginator('describe_instances')
+    sanitized_filters = dict()
+
     if ids:
         params = dict(InstanceIds=ids)
     elif filters is None:
@@ -1464,13 +1470,16 @@ def find_instances(ids=None, filters=None):
     else:
         for key in list(filters.keys()):
             if not key.startswith("tag:"):
-                filters[key.replace("_", "-")] = filters.pop(key)
-        params = dict(Filters=ansible_dict_to_boto3_filter_list(filters))
+                sanitized_filters[key.replace("_", "-")] = filters[key]
+            else:
+                sanitized_filters[key] = filters[key]
+        params = dict(Filters=ansible_dict_to_boto3_filter_list(sanitized_filters))
 
     try:
         results = _describe_instances(**params)
     except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
         module.fail_json_aws(e, msg="Could not describe instances")
+
     retval = list(results)
     return retval
 
