@@ -13,7 +13,7 @@ version_added: 1.0.0
 short_description: Manage AWS IAM users
 description:
   - A module to manage AWS IAM users.
-  - The module does not manage groups that users belong to, groups memberships can be managed using `iam_group`.
+  - The module does not manage groups that users belong to, groups memberships can be managed using M(community.aws.iam_group).
 author: Josh Souza (@joshsouza)
 options:
   name:
@@ -97,7 +97,7 @@ extends_documentation_fragment:
 EXAMPLES = r'''
 # Note: These examples do not set authentication details, see the AWS Guide for details.
 # Note: This module does not allow management of groups that users belong to.
-#       Groups should manage their membership directly using `iam_group`,
+#       Groups should manage their membership directly using community.aws.iam_group,
 #       as users belong to them.
 
 - name: Create a user
