@@ -232,7 +232,7 @@ EXAMPLES = r"""
 """
 
 RETURN = r"""
-# Digital Ocean API info https://developers.digitalocean.com/documentation/v2/#droplets
+# Digital Ocean API info https://docs.digitalocean.com/reference/api/api-reference/#tag/Droplets
 data:
     description: a DigitalOcean Droplet
     returned: changed
@@ -300,7 +300,7 @@ class DODroplet(object):
         "empty_response": "Empty response from the DigitalOcean API; please try again or open a bug if it never "
         "succeeds.",
         "resizing_off": "Droplet must be off prior to resizing: "
-        "https://developers.digitalocean.com/documentation/v2/#resize-a-droplet",
+        "https://docs.digitalocean.com/reference/api/api-reference/#operation/post_droplet_action",
         "unexpected": "Unexpected error [{0}]; please file a bug: "
         "https://github.com/ansible-collections/community.digitalocean/issues",
         "support_action": "Error status on Droplet action [{0}], please try again or contact DigitalOcean support: "
@@ -364,6 +364,7 @@ class DODroplet(object):
         return None
 
     def add_droplet_to_firewalls(self):
+        changed = False
         rule = self.get_firewall_by_name()
         if rule is None:
             err = "Failed to find firewalls: {0}".format(self.module.params["firewall"])
@@ -386,10 +387,12 @@ class DODroplet(object):
                         err = "Failed to add droplet {0} to firewall {1}".format(
                             droplet_id, rule[firewall]["id"]
                         )
-                        return err
-        return None
+                        return err, changed
+                    changed = True
+        return None, changed
 
     def remove_droplet_from_firewalls(self):
+        changed = False
         json_data = self.get_droplet()
         if json_data is not None:
             request_params = {}
@@ -408,11 +411,12 @@ class DODroplet(object):
                     json_data = response.json
                     status_code = response.status_code
                     if status_code != 204:
-                        err = "Failed to add droplet {0} to firewall {1}".format(
+                        err = "Failed to remove droplet {0} from firewall {1}".format(
                             droplet_id, firewall["id"]
                         )
-                        return err
-        return None
+                        return err, changed
+                    changed = True
+        return None, changed
 
     def get_by_id(self, droplet_id):
         if not droplet_id:
@@ -671,23 +675,26 @@ class DODroplet(object):
 
             # Add droplet to a firewall if specified
             if self.module.params["firewall"] is not None:
+                firewall_changed = False
                 if len(self.module.params["firewall"]) > 0:
-                    firewall_add = self.add_droplet_to_firewalls()
+                    firewall_add, add_changed = self.add_droplet_to_firewalls()
                     if firewall_add is not None:
                         self.module.fail_json(
                             changed=False,
                             msg=firewall_add,
                             data={"droplet": droplet, "firewall": firewall_add},
                         )
-                firewall_remove = self.remove_droplet_from_firewalls()
+                    firewall_changed = firewall_changed or add_changed
+                firewall_remove, remove_changed = self.remove_droplet_from_firewalls()
                 if firewall_remove is not None:
                     self.module.fail_json(
                         changed=False,
                         msg=firewall_remove,
                         data={"droplet": droplet, "firewall": firewall_remove},
                     )
+                firewall_changed = firewall_changed or remove_changed
                 self.module.exit_json(
-                    changed=True,
+                    changed=firewall_changed,
                     data={"droplet": droplet},
                 )
 
@@ -769,8 +776,7 @@ class DODroplet(object):
 
         # Get updated Droplet data (fallback to current data)
         if self.wait:
-            json_data = self.get_droplet()
-            # Without unique_name json_data is None
+            json_data = self.get_by_id(droplet_id)
             if json_data:
                 droplet = json_data.get("droplet", droplet)
 
@@ -809,9 +815,13 @@ class DODroplet(object):
         self.module.exit_json(changed=True, data={"droplet": droplet})
 
     def delete(self):
-        if not self.unique_name:
+        # to delete a droplet we need to know the droplet id or unique name, ie
+        # name is not None and unique_name is True, but as "id or name" is
+        # enforced elsewhere, we only need to enforce "id or unique_name" here
+        if not self.module.params["id"] and not self.unique_name:
             self.module.fail_json(
-                changed=False, msg="unique_name must be set for deletes"
+                changed=False,
+                msg="id must be set or unique_name must be true for deletes",
             )
         json_data = self.get_droplet()
         if json_data is None:
@@ -896,7 +906,6 @@ def main():
                 ("state", "present", ["name", "size", "image", "region"]),
                 ("state", "active", ["name", "size", "image", "region"]),
                 ("state", "inactive", ["name", "size", "image", "region"]),
-                ("state", "absent", ["name", "unique_name"]),
             ]
         ),
         supports_check_mode=True,
