@@ -157,8 +157,8 @@ options:
         version_added: 2.0.0
     dns4:
         description:
-            - A list of up to 3 dns servers.
-            - IPv4 format e.g. to add two IPv4 DNS server addresses, use C(192.0.2.53 198.51.100.53).
+            - A list of up to 3 DNS servers.
+            - The entries must be IPv4 addresses, for example C(192.0.2.53).
         elements: str
         type: list
     dns4_search:
@@ -255,8 +255,8 @@ options:
         version_added: 4.4.0
     dns6:
         description:
-            - A list of up to 3 dns servers.
-            - IPv6 format e.g. to add two IPv6 DNS server addresses, use C(2001:4860:4860::8888 2001:4860:4860::8844).
+            - A list of up to 3 DNS servers.
+            - The entries must be IPv6 addresses, for example C(2001:4860:4860::8888).
         elements: str
         type: list
     dns6_search:
@@ -374,8 +374,9 @@ options:
         description:
             - This is only used with 'bridge-slave' - 'hairpin mode' for the slave, which allows frames to be sent back out through the slave the
               frame was received on.
+            - The default value is C(true), but that is being deprecated
+              and it will be changed to C(false) in community.general 7.0.0.
         type: bool
-        default: yes
     runner:
         description:
             - This is the type of device or network connection that you wish to create for a team.
@@ -1376,7 +1377,8 @@ class Nmcli(object):
         self.hellotime = module.params['hellotime']
         self.maxage = module.params['maxage']
         self.ageingtime = module.params['ageingtime']
-        self.hairpin = module.params['hairpin']
+        # hairpin should be back to normal in 7.0.0
+        self._hairpin = module.params['hairpin']
         self.path_cost = module.params['path_cost']
         self.mac = module.params['mac']
         self.runner = module.params['runner']
@@ -1423,6 +1425,18 @@ class Nmcli(object):
 
         self.edit_commands = []
 
+    @property
+    def hairpin(self):
+        if self._hairpin is None:
+            self.module.deprecate(
+                "Parameter 'hairpin' default value will change from true to false in community.general 7.0.0. "
+                "Set the value explicitly to supress this warning.",
+                version='7.0.0', collection_name='community.general',
+            )
+            # Should be False in 7.0.0 but then that should be in argument_specs
+            self._hairpin = True
+        return self._hairpin
+
     def execute_command(self, cmd, use_unsafe_shell=False, data=None):
         if isinstance(cmd, list):
             cmd = [to_text(item) for item in cmd]
@@ -1728,7 +1742,10 @@ class Nmcli(object):
 
     @staticmethod
     def list_to_string(lst):
-        return ",".join(lst or [""])
+        if lst is None:
+            return None
+        else:
+            return ",".join(lst)
 
     @staticmethod
     def settings_type(setting):
@@ -2119,7 +2136,7 @@ def main():
             hellotime=dict(type='int', default=2),
             maxage=dict(type='int', default=20),
             ageingtime=dict(type='int', default=300),
-            hairpin=dict(type='bool', default=True),
+            hairpin=dict(type='bool'),
             path_cost=dict(type='int', default=100),
             # team specific vars
             runner=dict(type='str', default='roundrobin',
