@@ -47,7 +47,7 @@ try:
 except ImportError:
     ansible_version = 'unknown'
 
-COLLECTION_VERSION = "21.17.3"
+COLLECTION_VERSION = "21.20.0"
 CLIENT_APP_VERSION = "%s/%s" % ("%s", COLLECTION_VERSION)
 IMPORT_EXCEPTION = None
 
@@ -179,6 +179,7 @@ def get_feature(module, feature_name):
         # for SVM, whch protocols can be allowed
         svm_allowable_protocols_rest=['cifs', 'fcp', 'iscsi', 'nvme', 'nfs'],
         svm_allowable_protocols_zapi=['cifs', 'fcp', 'iscsi', 'nvme', 'nfs', 'ndmp', 'http'],
+        max_files_change_threshold=1,           # percentage of increase/decrease required to trigger a modify action
         warn_or_fail_on_fabricpool_backend_change='fail',
         no_cserver_ems=False                    # when True, don't attempt to find cserver and don't send cserver EMS
     )
@@ -190,17 +191,32 @@ def get_feature(module, feature_name):
     module.fail_json(msg="Internal error: unexpected feature flag: %s" % feature_name)
 
 
-def create_sf_connection(module, port=None):
-    hostname = module.params['hostname']
-    username = module.params['username']
-    password = module.params['password']
+def create_sf_connection(module, port=None, host_options=None):
+    if not HAS_SF_SDK:
+        module.fail_json(msg="the python SolidFire SDK module is required")
 
-    if HAS_SF_SDK and hostname and username and password:
-        try:
-            return ElementFactory.create(hostname, username, password, port=port)
-        except Exception as exc:
-            raise Exception("Unable to create SF connection: %s" % exc)
-    module.fail_json(msg="the python SolidFire SDK module is required")
+    if host_options is None:
+        host_options = module.params
+    msg, msg2 = None, None
+    missing_options = [option for option in ('hostname', 'username', 'password') if not host_options.get(option)]
+    if missing_options:
+        verb = 'are' if len(missing_options) > 1 else 'is'
+        msg = "%s %s required for ElementSW connection." % (', '.join(missing_options), verb)
+    extra_options = [option for option in ('cert_filepath', 'key_filepath') if host_options.get(option)]
+    if extra_options:
+        verb = 'are' if len(extra_options) > 1 else 'is'
+        msg2 = "%s %s not supported for ElementSW connection." % (', '.join(extra_options), verb)
+    msg = ("%s  %s") % (msg, msg2) if msg and msg2 else msg or msg2
+    if msg:
+        module.fail_json(msg=msg)
+    hostname = host_options.get('hostname')
+    username = host_options.get('username')
+    password = host_options.get('password')
+
+    try:
+        return ElementFactory.create(hostname, username, password, port=port)
+    except Exception as exc:
+        raise Exception("Unable to create SF connection: %s" % exc)
 
 
 def set_auth_method(module, username, password, cert_filepath, key_filepath):
@@ -657,7 +673,7 @@ class OntapRestAPI(object):
             self.module.fail_json(msg='  '.join(msgs))
         version = self.get_ontap_version()
         if version < (minimum_generation, minimum_major, minimum_minor):
-            msg = 'Error: ' + self.requires_ontap_version(module_name, '%d.%d' % (minimum_generation, minimum_major))
+            msg = 'Error: ' + self.requires_ontap_version(module_name, '%d.%d.%d' % (minimum_generation, minimum_major, minimum_minor))
             msg += '  Found: %s.%s.%s.' % version
             self.module.fail_json(msg=msg)
 
@@ -774,6 +790,7 @@ class OntapRestAPI(object):
         #     }
         # }
         error = None
+        errors = []
         message = None
         runtime = 0
         retries = 0
@@ -784,9 +801,10 @@ class OntapRestAPI(object):
             job_state = job_json.get('state', None) if job_json else None
             # ignore error if status is provided in the job
             if job_error and job_state is None:
-                error = job_error
+                errors.append(str(job_error))
                 retries += 1
                 if retries > max_retries:
+                    error = " - ".join(errors)
                     self.log_error(0, 'Job error: Reach max retries.')
                     break
             else:
@@ -830,7 +848,7 @@ class OntapRestAPI(object):
 
     def get(self, api, params=None, headers=None):
         method = 'GET'
-        dummy, message, error = self.send_request(method, api, params, headers=headers)
+        dummy, message, error = self.send_request(method, api, params, json=None, headers=headers)
         return message, error
 
     def post(self, api, body, params=None, headers=None):
@@ -850,7 +868,7 @@ class OntapRestAPI(object):
 
     def options(self, api, params=None, headers=None):
         method = 'OPTIONS'
-        dummy, message, error = self.send_request(method, api, params, headers=headers)
+        dummy, message, error = self.send_request(method, api, params, json=None, headers=headers)
         return message, error
 
     def set_version(self, message):
@@ -921,7 +939,7 @@ class OntapRestAPI(object):
                 "Minimum version of ONTAP for %s is %s\n"
                 % (property[0], str(property[1]))
                 for property in partially_supported_rest_properties
-                if self.get_ontap_version()[0:2] < property[1] and property[0] in parameters
+                if self.get_ontap_version()[0:3] < property[1] and property[0] in parameters
             )
             if error != '':
                 return True, error
@@ -934,6 +952,13 @@ class OntapRestAPI(object):
             if self.get_ontap_version()[:2] > (9, 5):
                 self.module.warn('Falling back to ZAPI because of unsupported option(s) or option value(s) in REST: %s' % used_unsupported_rest_properties)
             return False, None
+        if partially_supported_rest_properties:
+            # if ontap version is lower than partially_supported_rest_properties version, force ZAPI, only if the paramater is used
+            for property in partially_supported_rest_properties:
+                if self.get_ontap_version()[0:3] < property[1] and property[0] in parameters:
+                    self.module.warn(
+                        'Falling back to ZAPI because of unsupported option(s) or option value(s) "%s" in REST require %s' % (property[0], str(property[1])))
+                    return False, None
         if self.get_ontap_version()[:2] in ((9, 4), (9, 5)):
             # we can't trust REST support on 9.5, and not at all on 9.4
             return False, None
@@ -953,7 +978,7 @@ class OntapRestAPI(object):
     def is_rest(self, used_unsupported_rest_properties=None, partially_supported_rest_properties=None, parameters=None):
         ''' only return error if there is a reason to '''
         use_rest, error = self._is_rest(used_unsupported_rest_properties, partially_supported_rest_properties, parameters)
-        if used_unsupported_rest_properties is None:
+        if used_unsupported_rest_properties is None and partially_supported_rest_properties is None:
             return use_rest
         return use_rest, error
 
