@@ -21,12 +21,12 @@ version_added: 2.6.0
 author: NetApp Ansible Team (@carchi8py) <ng-ansibleteam@netapp.com>
 
 description:
-- Create or delete or modify export rules in ONTAP
+  - Create or delete or modify export rules in ONTAP
 
 options:
   state:
     description:
-    - Whether the specified export policy rule should exist or not.
+      - Whether the specified export policy rule should exist or not.
     required: false
     choices: ['present', 'absent']
     type: str
@@ -34,73 +34,84 @@ options:
 
   name:
     description:
-    - The name of the export policy this rule will be added to (or modified, or removed from).
+      - The name of the export policy this rule will be added to (or modified, or removed from).
     required: True
     type: str
     aliases:
-    - policy_name
+      - policy_name
 
   client_match:
     description:
-    - List of Client Match host names, IP Addresses, Netgroups, or Domains
-    - If rule_index is not provided, client_match is used as a key to fetch current rule to determine create,delete,modify actions.
-      If a rule with provided client_match exists, a new rule will not be created, but the existing rule will be modified or deleted.
-      If a rule with provided client_match doesn't exist, a new rule will be created if state is present.
+      - List of Client Match host names, IP Addresses, Netgroups, or Domains.
     type: list
     elements: str
 
   anonymous_user_id:
     description:
-    - User name or ID to which anonymous users are mapped. Default value is '65534'.
+      - User name or ID to which anonymous users are mapped. Default value is '65534'.
     type: str
 
   ro_rule:
     description:
-    - List of Read only access specifications for the rule
+      - List of Read only access specifications for the rule
     choices: ['any','none','never','krb5','krb5i','krb5p','ntlm','sys']
     type: list
     elements: str
 
   rw_rule:
     description:
-    - List of Read Write access specifications for the rule
+      - List of Read Write access specifications for the rule
     choices: ['any','none','never','krb5','krb5i','krb5p','ntlm','sys']
     type: list
     elements: str
 
   super_user_security:
     description:
-    - List of Read Write access specifications for the rule
+      - List of Read Write access specifications for the rule
     choices: ['any','none','never','krb5','krb5i','krb5p','ntlm','sys']
     type: list
     elements: str
 
   allow_suid:
     description:
-    - If 'true', NFS server will honor SetUID bits in SETATTR operation. Default value on creation is 'true'
+      - If 'true', NFS server will honor SetUID bits in SETATTR operation. Default value on creation is 'true'
     type: bool
 
   protocol:
     description:
-    - List of Client access protocols.
-    - Default value is set to 'any' during create.
+      - List of Client access protocols.
+      - Default value is set to 'any' during create.
     choices: [any,nfs,nfs3,nfs4,cifs,flexcache]
     type: list
     elements: str
     aliases:
-    - protocols
+      - protocols
 
   rule_index:
     description:
-    - index of the export policy rule
+      - index of the export policy rule
+    required: true
     type: int
 
+  from_rule_index:
+    description:
+      - index of the export policy rule to be re-indexed
+    type: int
+    version_added: 21.20.0
+
   vserver:
     description:
-    - Name of the vserver to use.
+      - Name of the vserver to use.
     required: true
     type: str
 
+  ntfs_unix_security:
+    description:
+      - NTFS export UNIX security options.
+      - With REST, supported from ONTAP 9.9.1 version.
+    type: str
+    choices: ['fail', 'ignore']
+    version_added: 21.18.0
 '''
 
 EXAMPLES = """
@@ -108,6 +119,7 @@ EXAMPLES = """
       netapp.ontap.na_ontap_export_policy_rule:
         state: present
         name: default123
+        rule_index: 100
         vserver: ci_dev
         client_match: 0.0.0.0/0,1.1.1.0/24
         ro_rule: krb5,krb5i
@@ -116,6 +128,7 @@ EXAMPLES = """
         super_user_security: any
         anonymous_user_id: 65534
         allow_suid: true
+        ntfs_unix_security: ignore
         hostname: "{{ netapp_hostname }}"
         username: "{{ netapp_username }}"
         password: "{{ netapp_password }}"
@@ -131,6 +144,24 @@ EXAMPLES = """
         rw_rule: any
         protocol: any
         allow_suid: false
+        ntfs_unix_security: fail
+        hostname: "{{ netapp_hostname }}"
+        username: "{{ netapp_username }}"
+        password: "{{ netapp_password }}"
+
+    - name: rename ExportPolicyRule index
+      netapp.ontap.na_ontap_export_policy_rule:
+        state: present
+        name: default123
+        from_rule_index: 100
+        rule_index: 99
+        client_match: 0.0.0.0/0
+        anonymous_user_id: 65521
+        ro_rule: ntlm
+        rw_rule: any
+        protocol: any
+        allow_suid: false
+        ntfs_unix_security: fail
         hostname: "{{ netapp_hostname }}"
         username: "{{ netapp_username }}"
         password: "{{ netapp_password }}"
@@ -139,7 +170,7 @@ EXAMPLES = """
       netapp.ontap.na_ontap_export_policy_rule:
         state: absent
         name: default123
-        rule_index: 100
+        rule_index: 99
         vserver: ci_dev
         hostname: "{{ netapp_hostname }}"
         username: "{{ netapp_username }}"
@@ -184,9 +215,11 @@ class NetAppontapExportRule:
                                      type='list', elements='str', default=None,
                                      choices=['any', 'none', 'never', 'krb5', 'krb5i', 'krb5p', 'ntlm', 'sys']),
             allow_suid=dict(required=False, type='bool'),
-            rule_index=dict(required=False, type='int'),
+            from_rule_index=dict(required=False, type='int'),
+            rule_index=dict(required=True, type='int'),
             anonymous_user_id=dict(required=False, type='str'),
             vserver=dict(required=True, type='str'),
+            ntfs_unix_security=dict(required=False, type='str', choices=['fail', 'ignore'])
         ))
 
         self.module = AnsibleModule(
@@ -200,21 +233,19 @@ class NetAppontapExportRule:
 
         unsupported_rest_properties = ['allow_suid']
         self.rest_api = OntapRestAPI(self.module)
-        used_unsupported_rest_properties = [x for x in unsupported_rest_properties if x in self.parameters]
-        self.use_rest, error = self.rest_api.is_rest(used_unsupported_rest_properties)
-        if error:
-            self.module.fail_json(msg=error)
-        elif not netapp_utils.has_netapp_lib():
-            self.module.fail_json(msg=netapp_utils.netapp_lib_is_required())
-        else:
-            self.server = netapp_utils.setup_na_ontap_zapi(
-                module=self.module, vserver=self.parameters['vserver'])
+        partially_supported_rest_properties = [['ntfs_unix_security', (9, 9, 1)]]
+        self.use_rest = self.rest_api.is_rest_supported_properties(self.parameters, unsupported_rest_properties, partially_supported_rest_properties)
+        if not self.use_rest:
+            if not netapp_utils.has_netapp_lib():
+                self.module.fail_json(msg=netapp_utils.netapp_lib_is_required())
+            self.server = netapp_utils.setup_na_ontap_zapi(module=self.module, vserver=self.parameters['vserver'])
 
     def set_playbook_zapi_key_map(self):
         self.na_helper.zapi_string_keys = {
             'anonymous_user_id': 'anonymous-user-id',
             'client_match': 'client-match',
-            'name': 'policy-name'
+            'name': 'policy-name',
+            'ntfs_unix_security': 'export-ntfs-unix-security-ops'
         }
         self.na_helper.zapi_list_keys = {
             'protocol': ('protocol', 'access-protocol'),
@@ -229,7 +260,7 @@ class NetAppontapExportRule:
             'rule_index': 'rule-index'
         }
 
-    def set_query_parameters(self):
+    def set_query_parameters(self, rule_index):
         """
         Return dictionary of query parameters and
         :return:
@@ -238,22 +269,14 @@ class NetAppontapExportRule:
             'policy-name': self.parameters['name'],
             'vserver': self.parameters['vserver']
         }
-
-        if self.parameters.get('rule_index'):
-            query['rule-index'] = self.parameters['rule_index']
-        elif self.parameters.get('client_match'):
-            query['client-match'] = self.parameters['client_match']
-        else:
-            self.module.fail_json(
-                msg="Need to specify at least one of the rule_index and client_match option.")
-
+        query['rule-index'] = rule_index
         return {
             'query': {
                 'export-rule-info': query
             }
         }
 
-    def get_export_policy_rule(self):
+    def get_export_policy_rule(self, rule_index):
         """
         Return details about the export policy rule
         :param:
@@ -262,10 +285,10 @@ class NetAppontapExportRule:
         :rtype: dict
         """
         if self.use_rest:
-            return self.get_export_policy_rule_rest()
+            return self.get_export_policy_rule_rest(rule_index)
         current, result = None, None
         rule_iter = netapp_utils.zapi.NaElement('export-rule-get-iter')
-        rule_iter.translate_struct(self.set_query_parameters())
+        rule_iter.translate_struct(self.set_query_parameters(rule_index))
         try:
             result = self.server.invoke_successfully(rule_iter, True)
         except netapp_utils.zapi.NaApiError as error:
@@ -289,8 +312,6 @@ class NetAppontapExportRule:
                 current[item_key] = self.na_helper.get_value_for_list(from_zapi=True,
                                                                       zapi_parent=rule_info.get_child_by_name(parent))
             current['num_records'] = int(result.get_child_content('num-records'))
-            if not self.parameters.get('rule_index'):
-                self.parameters['rule_index'] = current['rule_index']
         return current
 
     def get_export_policy(self):
@@ -404,24 +425,35 @@ class NetAppontapExportRule:
                                   % (self.parameters['name'], to_native(error)),
                                   exception=traceback.format_exc())
 
-    def modify_export_policy_rule(self, params, policy_id=None, rule_index=None):
+    def modify_export_policy_rule(self, params, policy_id=None, rule_index=None, rename=False):
         '''
         Modify an existing export policy rule
         :param params: dict() of attributes with desired values
         :return: None
         '''
         if self.use_rest:
-            return self.modify_export_policy_rule_rest(params, policy_id['id'], rule_index)
+            return self.modify_export_policy_rule_rest(params, policy_id['id'], rule_index, rename)
         export_rule_modify = netapp_utils.zapi.NaElement.create_node_with_children(
             'export-rule-modify', **{'policy-name': self.parameters['name'],
-                                     'rule-index': str(self.parameters['rule_index'])})
+                                     'rule-index': str(rule_index)})
         self.add_parameters_for_create_or_modify(export_rule_modify, params)
         try:
             self.server.invoke_successfully(export_rule_modify, enable_tunneling=True)
         except netapp_utils.zapi.NaApiError as error:
-            self.module.fail_json(msg='Error modifying allow_suid %s: %s'
-                                  % (self.parameters['allow_suid'], to_native(error)),
+            self.module.fail_json(msg='Error modifying rule index %s: %s'
+                                  % (rule_index, to_native(error)),
                                   exception=traceback.format_exc())
+        if rename:
+            export_rule_set_index = netapp_utils.zapi.NaElement.create_node_with_children(
+                'export-rule-set-index', **{'policy-name': self.parameters['name'],
+                                            'rule-index': str(self.parameters['from_rule_index']),
+                                            'new-rule-index': str(self.parameters['rule_index'])})
+            try:
+                self.server.invoke_successfully(export_rule_set_index, enable_tunneling=True)
+            except netapp_utils.zapi.NaApiError as error:
+                self.module.fail_json(msg='Error reindex rule index %s: %s'
+                                          % (self.parameters['from_rule_index'], to_native(error)),
+                                      exception=traceback.format_exc())
 
     def autosupport_log(self):
         netapp_utils.ems_log_event("na_ontap_export_policy_rules", self.server)
@@ -436,14 +468,26 @@ class NetAppontapExportRule:
             self.module.fail_json(msg="Error on fetching export policy: %s" % error)
         return record
 
-    def get_export_policy_rule_rest(self):
+    def get_export_policy_rules_rest(self):
+        policy = self.get_export_policy_rest()
+        if not policy:
+            return None
+        api = 'protocols/nfs/export-policies/%s/rules' % (policy['id'])
+        records, error = rest_generic.get_0_or_more_records(self.rest_api, api)
+        if error:
+            self.module.fail_json(msg="Error on fetching export policy rule: %s" % error)
+        if not records:
+            return None
+        return self.filter_get_results(records['records'])
+
+    def get_export_policy_rule_rest(self, rule_index):
         policy = self.get_export_policy_rest()
         if not policy:
             return None
-        if self.parameters.get('rule_index') is None:
-            return self.get_export_policy_rule_without_index(policy)
         options = {'fields': 'anonymous_user,clients,index,protocols,ro_rule,rw_rule,superuser'}
-        api = 'protocols/nfs/export-policies/%s/rules/%s' % (policy['id'], self.parameters['rule_index'])
+        if 'ntfs_unix_security' in self.parameters:
+            options['fields'] += ',ntfs_unix_security'
+        api = 'protocols/nfs/export-policies/%s/rules/%s' % (policy['id'], rule_index)
         record, error = rest_generic.get_one_record(self.rest_api, api, options)
         if error:
             # If rule index passed in doesn't exist, return None
@@ -454,28 +498,6 @@ class NetAppontapExportRule:
             return None
         return self.filter_get_results(record)
 
-    def get_export_policy_rule_without_index(self, policy):
-        options = {'fields': 'anonymous_user,clients,index,protocols,ro_rule,rw_rule,superuser'}
-        api = 'protocols/nfs/export-policies/%s/rules' % policy['id']
-        records, error = rest_generic.get_0_or_more_records(self.rest_api, api, options)
-        if error:
-            self.module.fail_json(msg="Error on fetching export policy rule: %s" % error)
-        if not records:
-            return None
-        return_records = {}
-        for record in records:
-            for match in record.get('clients', []):
-                if match['match'] in self.parameters['client_match']:
-                    return_records[record['index']] = record
-        if not return_records:
-            return None
-        if len(return_records) > 1:
-            self.module.fail_json(msg="More than one export policy rule match client match: %s found the following %s" %
-                                      (self.parameters['client_match'],
-                                       str(return_records)))
-        record = self.filter_get_results(list(return_records.values())[0])
-        return record
-
     def filter_get_results(self, record):
         record['rule_index'] = record.pop('index')
         record['anonymous_user_id'] = record.pop('anonymous_user')
@@ -502,10 +524,17 @@ class NetAppontapExportRule:
             body['protocols'] = self.parameters['protocol']
         if self.parameters.get('super_user_security'):
             body['superuser'] = self.parameters['super_user_security']
-        api = 'protocols/nfs/export-policies/%s/rules' % str(policy_id)
-        dummy, error = rest_generic.post_async(self.rest_api, api, body)
+        if self.parameters.get('ntfs_unix_security'):
+            body['ntfs_unix_security'] = self.parameters['ntfs_unix_security']
+        api = 'protocols/nfs/export-policies/%s/rules?return_records=true' % str(policy_id)
+        response, error = rest_generic.post_async(self.rest_api, api, body)
         if error:
             self.module.fail_json(msg="Error on creating export policy Rule: %s" % error)
+        if response['num_records'] == 1:
+            rule_index = response['records'][0]['index']
+        else:
+            self.module.fail_json(msg="Error on creating export policy Rule, returned response is invalid: %s" % response)
+        self.modify_export_policy_rule_rest({}, policy_id, rule_index, True)
 
     def client_match_format(self, client_match):
         client_match = client_match.split(',')
@@ -517,7 +546,7 @@ class NetAppontapExportRule:
         if error:
             self.module.fail_json(msg="Error on deleting export policy Rule: %s" % error)
 
-    def modify_export_policy_rule_rest(self, params, policy_id, rule_index):
+    def modify_export_policy_rule_rest(self, params, policy_id, rule_index, rename=False):
         body = {}
         if params.get('anonymous_user_id'):
             body['anonymous_user'] = self.parameters['anonymous_user_id']
@@ -531,8 +560,13 @@ class NetAppontapExportRule:
             body['ro_rule'] = self.parameters['ro_rule']
         if params.get('rw_rule'):
             body['rw_rule'] = self.parameters['rw_rule']
+        if params.get('ntfs_unix_security'):
+            body['ntfs_unix_security'] = self.parameters['ntfs_unix_security']
         api = 'protocols/nfs/export-policies/%s/rules' % str(policy_id)
-        dummy, error = rest_generic. patch_async(self.rest_api, api, rule_index, body)
+        query = None
+        if rename:
+            query = {'new_index': self.parameters['rule_index']}
+        dummy, error = rest_generic.patch_async(self.rest_api, api, rule_index, body, query)
         if error:
             self.module.fail_json(msg="Error on modifying export policy Rule: %s" % error)
 
@@ -544,26 +578,33 @@ class NetAppontapExportRule:
         if self.parameters.get('client_match') is not None:
             self.parameters['client_match'] = ','.join(self.parameters['client_match'])
             self.parameters['client_match'] = self.parameters['client_match'].replace(' ', '')
-        current, modify = self.get_export_policy_rule(), None
+        current = self.get_export_policy_rule(self.parameters['rule_index'])
         current_policy = self.get_export_policy()
-        # rule_index can be modified in Zapi, but can't be modified in REST
-        if current and self.use_rest:
-            current_filters = current.copy()
-            current_filters.pop('rule_index')
+        cd_action, rename, modify = None, None, None
+        cd_action = self.na_helper.get_cd_action(current, self.parameters)
+        if cd_action == 'create' and self.parameters.get('from_rule_index'):
+            from_current = self.get_export_policy_rule(self.parameters['from_rule_index'])
+            rename = self.na_helper.is_rename_action(from_current, current)
+            if rename is None:
+                self.module.fail_json(
+                    msg="Error renaming: export policy rule %s does not exist" % self.parameters['from_rule_index'])
+            if rename:
+                current = from_current
+                modify = self.na_helper.get_modified_attributes(current, self.parameters)
         else:
-            current_filters = current
-        action = self.na_helper.get_cd_action(current_filters, self.parameters)
-        if action is None and self.parameters['state'] == 'present':
-            modify = self.na_helper.get_modified_attributes(current, self.parameters)
+            if cd_action is None and self.parameters['state'] == 'present':
+                modify = self.na_helper.get_modified_attributes(current, self.parameters)
 
         if self.na_helper.changed and not self.module.check_mode:
             # create export policy (if policy doesn't exist) only when changed=True
-            if action == 'create':
+            if rename:
+                self.modify_export_policy_rule(modify, current_policy, self.parameters['from_rule_index'], rename=True)
+            elif cd_action == 'create':
                 if not current_policy:
                     self.create_export_policy()
                     current_policy = self.get_export_policy()
                 self.create_export_policy_rule(current_policy)
-            elif action == 'delete':
+            elif cd_action == 'delete':
                 if self.use_rest:
                     self.delete_export_policy_rule(current['rule_index'], current_policy)
                 elif current['num_records'] > 1:
