@@ -14,7 +14,7 @@ from ansible.module_utils import basic
 from ansible_collections.netapp.ontap.tests.unit.compat.mock import call, patch
 
 from ansible_collections.netapp.ontap.tests.unit.plugins.module_utils.ansible_mocks import \
-    patch_ansible, create_module, expect_and_capture_ansible_exception
+    assert_warning_was_raised, create_module, expect_and_capture_ansible_exception, patch_ansible
 from ansible_collections.netapp.ontap.tests.unit.framework.mock_rest_and_zapi_requests import patch_request_and_invoke, register_responses
 from ansible_collections.netapp.ontap.tests.unit.framework.rest_factory import rest_responses
 import ansible_collections.netapp.ontap.plugins.module_utils.netapp as netapp_utils
@@ -364,7 +364,7 @@ def test_fail_if_not_rest_minimum_version():
         'Error using REST for version, error: Expected error.  Error using REST for version, status_code: 400.'
     # version mismatch
     assert expect_and_capture_ansible_exception(rest_api.fail_if_not_rest_minimum_version, 'fail', 'module_name', 9, 7)['msg'] ==\
-        'Error: module_name only supports REST, and requires ONTAP 9.7 or later.  Found: 9.6.0.'
+        'Error: module_name only supports REST, and requires ONTAP 9.7.0 or later.  Found: 9.6.0.'
     # version match
     assert rest_api.fail_if_not_rest_minimum_version('module_name', 9, 6) is None
 
@@ -469,6 +469,18 @@ def test_is_rest_supported_properties():
     assert rest_api.is_rest_supported_properties(['abc'], ['xyz'])
 
 
+def test_is_rest_partially_supported_properties():
+    register_responses([
+        ('GET', 'cluster', SRR['is_rest_97']),
+        ('GET', 'cluster', SRR['is_rest_9_9_0']),
+    ])
+    rest_api = create_restapi_object(DEFAULT_ARGS)
+    rest_api.use_rest = 'auto'
+    assert not rest_api.is_rest_supported_properties(['xyz'], None, [('xyz', (9, 8, 1))])
+    assert_warning_was_raised('Falling back to ZAPI because of unsupported option(s) or option value(s) "xyz" in REST require (9, 8, 1)')
+    assert rest_api.is_rest_supported_properties(['xyz'], None, [('xyz', (9, 8, 1))])
+
+
 def test_is_rest():
     rest_api = create_restapi_object(DEFAULT_ARGS)
     # testing always with used_unsupported_rest_properties
